@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example.registration;
@@ -21,7 +21,7 @@ public class WebServerExtension implements BeforeAllCallback {
 	}
 
 	public String getServerUrl() {
-		return "http://example.org:8181";
+		return "https://example.org:8181";
 	}
 
 	public static Builder builder() {
