@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example;
@@ -16,7 +16,7 @@ import org.junit.jupiter.api.Test;
 
 class DisabledTestsDemo {
 
-	@Disabled
+	@Disabled("Disabled until bug #42 has been resolved")
 	@Test
 	void testWillBeSkipped() {
 	}
@@ -24,5 +24,6 @@ class DisabledTestsDemo {
 	@Test
 	void testWillBeExecuted() {
 	}
+
 }
 // end::user_guide[]
