@@ -1,15 +1,16 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example;
 
+import java.util.HashMap;
 import java.util.Map;
 
 import org.junit.jupiter.api.Test;
@@ -33,17 +34,11 @@ class TestReporterDemo {
 
 	@Test
 	void reportMultipleKeyValuePairs(TestReporter testReporter) {
-		// end::user_guide[]
-		// @formatter:off
-		// tag::user_guide[]
-		testReporter.publishEntry(
-			Map.of(
-				"user name", "dk38",
-				"award year", "1974"
-			));
-		// end::user_guide[]
-		// @formatter:on
-		// tag::user_guide[]
+		Map<String, String> values = new HashMap<>();
+		values.put("user name", "dk38");
+		values.put("award year", "1974");
+
+		testReporter.publishEntry(values);
 	}
 
 }
