@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example.testinterface;
@@ -15,15 +15,15 @@ import static org.junit.jupiter.api.Assertions.assertEquals;
 import org.junit.jupiter.api.Test;
 
 // @formatter:off
-//tag::user_guide[]
+// tag::user_guide[]
 class TestInterfaceDemo implements TestLifecycleLogger,
 		TimeExecutionLogger, TestInterfaceDynamicTestsDemo {
 
 	@Test
 	void isEqualValue() {
-		assertEquals(1, 1, "is always equal");
+		assertEquals(1, "a".length(), "is always equal");
 	}
 
 }
-//end::user_guide[]
+// end::user_guide[]
 // @formatter:on
