@@ -1,18 +1,16 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.jupiter.api;
 
-import static org.junit.jupiter.api.AssertionUtils.fail;
-import static org.junit.jupiter.api.AssertionUtils.format;
-import static org.junit.jupiter.api.AssertionUtils.nullSafeGet;
+import static org.junit.jupiter.api.AssertionFailureBuilder.assertionFailure;
 
 import java.util.function.Supplier;
 
@@ -40,12 +38,16 @@ class AssertSame {
 
 	static void assertSame(Object expected, Object actual, Supplier<String> messageSupplier) {
 		if (expected != actual) {
-			failNotSame(expected, actual, nullSafeGet(messageSupplier));
+			failNotSame(expected, actual, messageSupplier);
 		}
 	}
 
-	private static void failNotSame(Object expected, Object actual, String message) {
-		fail(format(expected, actual, message), expected, actual);
+	private static void failNotSame(Object expected, Object actual, Object messageOrSupplier) {
+		assertionFailure() //
+				.message(messageOrSupplier) //
+				.expected(expected) //
+				.actual(actual) //
+				.buildAndThrow();
 	}
 
 }
