@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.vintage.engine;
@@ -39,9 +39,4 @@ class VintageTestEngineBasicTests {
 		assertEquals("junit-vintage-engine", vintage.getArtifactId().get());
 	}
 
-	@Test
-	void version() {
-		assertEquals("DEVELOPMENT", vintage.getVersion().get());
-	}
-
 }
