@@ -1,16 +1,15 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.platform.commons.util;
 
-import static java.util.Arrays.asList;
 import static java.util.stream.Collectors.toList;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -37,13 +36,13 @@ import java.lang.annotation.RetentionPolicy;
 import java.lang.annotation.Target;
 import java.lang.reflect.AnnotatedElement;
 import java.lang.reflect.Field;
-import java.lang.reflect.Method;
 import java.math.BigDecimal;
 import java.util.List;
 import java.util.Optional;
 import java.util.function.Predicate;
 
 import org.junit.jupiter.api.Test;
+import org.junit.platform.commons.PreconditionViolationException;
 
 /**
  * Unit tests for {@link AnnotationUtils}.
@@ -128,22 +127,48 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotationDirectlyPresentOnMethod() throws Exception {
-		Method method = Annotation2Class.class.getDeclaredMethod("method");
+		var method = Annotation2Class.class.getDeclaredMethod("method");
 		assertThat(findAnnotation(method, Annotation1.class)).isPresent();
 	}
 
 	@Test
 	void findAnnotationMetaPresentOnMethod() throws Exception {
-		Method method = ComposedAnnotationClass.class.getDeclaredMethod("method");
+		var method = ComposedAnnotationClass.class.getDeclaredMethod("method");
 		assertThat(findAnnotation(method, Annotation1.class)).isPresent();
 	}
 
 	@Test
 	void findAnnotationMetaPresentOnOptionalMethod() throws Exception {
-		Method method = ComposedAnnotationClass.class.getDeclaredMethod("method");
+		var method = ComposedAnnotationClass.class.getDeclaredMethod("method");
 		assertThat(findAnnotation(Optional.of(method), Annotation1.class)).isPresent();
 	}
 
+	@Test
+	void findAnnotationDirectlyPresentOnEnclosingClass() throws Exception {
+		Class<?> clazz = Annotation1Class.InnerClass.class;
+		assertThat(findAnnotation(clazz, Annotation1.class, false)).isNotPresent();
+		assertThat(findAnnotation(clazz, Annotation1.class, true)).isPresent();
+
+		clazz = Annotation1Class.InnerClass.InnerInnerClass.class;
+		assertThat(findAnnotation(clazz, Annotation1.class, false)).isNotPresent();
+		assertThat(findAnnotation(clazz, Annotation1.class, true)).isPresent();
+
+		clazz = Annotation1Class.NestedClass.class;
+		assertThat(findAnnotation(clazz, Annotation1.class, false)).isNotPresent();
+		assertThat(findAnnotation(clazz, Annotation1.class, true)).isNotPresent();
+	}
+
+	@Test
+	void findAnnotationMetaPresentOnEnclosingClass() throws Exception {
+		Class<?> clazz = ComposedAnnotationClass.InnerClass.class;
+		assertThat(findAnnotation(clazz, Annotation1.class, false)).isNotPresent();
+		assertThat(findAnnotation(clazz, Annotation1.class, true)).isPresent();
+
+		clazz = ComposedAnnotationClass.InnerClass.InnerInnerClass.class;
+		assertThat(findAnnotation(clazz, Annotation1.class, false)).isNotPresent();
+		assertThat(findAnnotation(clazz, Annotation1.class, true)).isPresent();
+	}
+
 	@Test
 	void isAnnotatedForClassWithoutAnnotation() {
 		assertFalse(isAnnotated(Annotation1Class.class, Annotation2.class));
@@ -176,110 +201,128 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findRepeatableAnnotationsForNotRepeatableAnnotation() {
-		PreconditionViolationException exception = assertThrows(PreconditionViolationException.class,
+		var exception = assertThrows(PreconditionViolationException.class,
 			() -> findRepeatableAnnotations(getClass(), Inherited.class));
 
 		assertThat(exception.getMessage()).isEqualTo(Inherited.class.getName() + " must be @Repeatable");
 	}
 
 	@Test
-	void findRepeatableAnnotationsForNullAnnotatedElement() {
-		assertThat(findRepeatableAnnotations(null, Tag.class)).isEmpty();
+	void findRepeatableAnnotationsForNullOptionalAnnotatedElement() {
+		assertThat(findRepeatableAnnotations((Optional<AnnotatedElement>) null, Tag.class)).isEmpty();
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithSingleTag() throws Exception {
+	void findRepeatableAnnotationsForEmptyOptionalAnnotatedElement() {
+		assertThat(findRepeatableAnnotations(Optional.empty(), Tag.class)).isEmpty();
+	}
+
+	@Test
+	void findRepeatableAnnotationsForNullAnnotatedElement() {
+		assertThat(findRepeatableAnnotations((AnnotatedElement) null, Tag.class)).isEmpty();
+	}
+
+	@Test
+	void findRepeatableAnnotationsWithSingleTag() {
 		assertTagsFound(SingleTaggedClass.class, "a");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithSingleComposedTag() throws Exception {
+	void findRepeatableAnnotationsWithSingleComposedTag() {
 		assertTagsFound(SingleComposedTaggedClass.class, "fast");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithSingleComposedTagOnImplementedInterface() throws Exception {
+	void findRepeatableAnnotationsWithSingleComposedTagOnImplementedInterface() {
 		assertTagsFound(TaggedInterfaceClass.class, "fast");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithLocalComposedTagAndComposedTagOnImplementedInterface() throws Exception {
+	void findRepeatableAnnotationsWithLocalComposedTagAndComposedTagOnImplementedInterface() {
 		assertTagsFound(LocalTagOnTaggedInterfaceClass.class, "fast", "smoke");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithMultipleTags() throws Exception {
+	void findRepeatableAnnotationsWithMultipleTags() {
 		assertTagsFound(MultiTaggedClass.class, "a", "b", "c");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithMultipleComposedTags() throws Exception {
+	void findRepeatableAnnotationsWithMultipleComposedTags() {
 		assertTagsFound(MultiComposedTaggedClass.class, "fast", "smoke");
 		assertTagsFound(FastAndSmokyTaggedClass.class, "fast", "smoke");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithContainer() throws Exception {
+	void findRepeatableAnnotationsWithContainer() {
 		assertTagsFound(ContainerTaggedClass.class, "a", "b", "c", "d");
 	}
 
 	@Test
-	void findRepeatableAnnotationsWithComposedTagBeforeContainer() throws Exception {
+	void findRepeatableAnnotationsWithComposedTagBeforeContainer() {
 		assertTagsFound(ContainerAfterComposedTaggedClass.class, "fast", "a", "b", "c");
 	}
 
-	private void assertTagsFound(Class<?> clazz, String... tags) throws Exception {
-		assertEquals(asList(tags),
+	private void assertTagsFound(Class<?> clazz, String... tags) {
+		assertEquals(List.of(tags),
 			findRepeatableAnnotations(clazz, Tag.class).stream().map(Tag::value).collect(toList()),
 			() -> "Tags found for class " + clazz.getName());
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithSingleAnnotationOnSuperclass() throws Exception {
+	void findInheritedRepeatableAnnotationsWithSingleAnnotationOnSuperclass() {
 		assertExtensionsFound(SingleExtensionClass.class, "a");
 		assertExtensionsFound(SubSingleExtensionClass.class, "a");
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithMultipleAnnotationsOnSuperclass() throws Exception {
+	void findInheritedRepeatableAnnotationsWithMultipleAnnotationsOnSuperclass() {
 		assertExtensionsFound(MultiExtensionClass.class, "a", "b", "c");
 		assertExtensionsFound(SubMultiExtensionClass.class, "a", "b", "c", "x", "y", "z");
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithContainerAnnotationOnSuperclass() throws Exception {
+	void findInheritedRepeatableAnnotationsWithContainerAnnotationOnSuperclass() {
 		assertExtensionsFound(ContainerExtensionClass.class, "a", "b", "c");
 		assertExtensionsFound(SubContainerExtensionClass.class, "a", "b", "c", "x");
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithSingleComposedAnnotation() throws Exception {
+	void findInheritedRepeatableAnnotationsWithSingleComposedAnnotation() {
 		assertExtensionsFound(SingleComposedExtensionClass.class, "foo");
 	}
 
+	/**
+	 * @since 1.5
+	 */
 	@Test
-	void findInheritedRepeatableAnnotationsWithSingleComposedAnnotationOnSuperclass() throws Exception {
+	void findInheritedRepeatableAnnotationsWithComposedAnnotationsInNestedContainer() {
+		assertExtensionsFound(MultipleFoos1.class, "foo");
+		assertExtensionsFound(MultipleFoos2.class, "foo");
+	}
+
+	@Test
+	void findInheritedRepeatableAnnotationsWithSingleComposedAnnotationOnSuperclass() {
 		assertExtensionsFound(SubSingleComposedExtensionClass.class, "foo");
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithMultipleComposedAnnotations() throws Exception {
+	void findInheritedRepeatableAnnotationsWithMultipleComposedAnnotations() {
 		assertExtensionsFound(MultiComposedExtensionClass.class, "foo", "bar");
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithMultipleComposedAnnotationsOnSuperclass() throws Exception {
+	void findInheritedRepeatableAnnotationsWithMultipleComposedAnnotationsOnSuperclass() {
 		assertExtensionsFound(SubMultiComposedExtensionClass.class, "foo", "bar");
 	}
 
 	@Test
-	void findInheritedRepeatableAnnotationsWithMultipleComposedAnnotationsOnSuperclassAndLocalContainerAndComposed()
-			throws Exception {
+	void findInheritedRepeatableAnnotationsWithMultipleComposedAnnotationsOnSuperclassAndLocalContainerAndComposed() {
 		assertExtensionsFound(ContainerPlusSubMultiComposedExtensionClass.class, "foo", "bar", "x", "y", "z");
 	}
 
-	private void assertExtensionsFound(Class<?> clazz, String... tags) throws Exception {
-		assertEquals(asList(tags),
+	private void assertExtensionsFound(Class<?> clazz, String... tags) {
+		assertEquals(List.of(tags),
 			findRepeatableAnnotations(clazz, ExtendWith.class).stream().map(ExtendWith::value).collect(toList()),
 			() -> "Extensions found for class " + clazz.getName());
 	}
@@ -303,21 +346,21 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotatedMethodsForAnnotationOnMethodsInClassUsingHierarchyDownMode() throws Exception {
-		Method method2 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method2");
-		Method method3 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method3");
+		var method2 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method2");
+		var method3 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method3");
 
-		List<Method> methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation2.class, TOP_DOWN);
+		var methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation2.class, TOP_DOWN);
 
 		assertThat(methods).containsOnly(method2, method3);
 	}
 
 	@Test
 	void findAnnotatedMethodsForAnnotationOnMethodsInClassHierarchyUsingHierarchyUpMode() throws Exception {
-		Method method1 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method1");
-		Method method3 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method3");
-		Method superMethod = SuperclassWithAnnotatedMethod.class.getDeclaredMethod("superMethod");
+		var method1 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method1");
+		var method3 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method3");
+		var superMethod = SuperclassWithAnnotatedMethod.class.getDeclaredMethod("superMethod");
 
-		List<Method> methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation1.class, BOTTOM_UP);
+		var methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation1.class, BOTTOM_UP);
 
 		assertEquals(3, methods.size());
 		assertThat(methods.subList(0, 2)).containsOnly(method1, method3);
@@ -326,11 +369,11 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotatedMethodsForAnnotationUsedInClassAndSuperclassHierarchyDown() throws Exception {
-		Method method1 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method1");
-		Method method3 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method3");
-		Method superMethod = SuperclassWithAnnotatedMethod.class.getDeclaredMethod("superMethod");
+		var method1 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method1");
+		var method3 = ClassWithAnnotatedMethods.class.getDeclaredMethod("method3");
+		var superMethod = SuperclassWithAnnotatedMethod.class.getDeclaredMethod("superMethod");
 
-		List<Method> methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation1.class, TOP_DOWN);
+		var methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation1.class, TOP_DOWN);
 
 		assertEquals(3, methods.size());
 		assertEquals(superMethod, methods.get(0));
@@ -339,9 +382,9 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotatedMethodsForAnnotationUsedInInterface() throws Exception {
-		Method interfaceMethod = InterfaceWithAnnotatedDefaultMethod.class.getDeclaredMethod("interfaceMethod");
+		var interfaceMethod = InterfaceWithAnnotatedDefaultMethod.class.getDeclaredMethod("interfaceMethod");
 
-		List<Method> methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation3.class, BOTTOM_UP);
+		var methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation3.class, BOTTOM_UP);
 
 		assertThat(methods).containsExactly(interfaceMethod);
 	}
@@ -373,23 +416,21 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotatedFieldsForAnnotationOnFieldsInClassUsingHierarchyDownMode() throws Exception {
-		Field field2 = ClassWithAnnotatedFields.class.getDeclaredField("field2");
-		Field field3 = ClassWithAnnotatedFields.class.getDeclaredField("field3");
+		var field2 = ClassWithAnnotatedFields.class.getDeclaredField("field2");
+		var field3 = ClassWithAnnotatedFields.class.getDeclaredField("field3");
 
-		List<Field> fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation2.class, isStringField,
-			TOP_DOWN);
+		var fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation2.class, isStringField, TOP_DOWN);
 
 		assertThat(fields).containsOnly(field2, field3);
 	}
 
 	@Test
 	void findAnnotatedFieldsForAnnotationOnFieldsInClassHierarchyUsingHierarchyUpMode() throws Exception {
-		Field field1 = ClassWithAnnotatedFields.class.getDeclaredField("field1");
-		Field field3 = ClassWithAnnotatedFields.class.getDeclaredField("field3");
-		Field superField = SuperclassWithAnnotatedField.class.getDeclaredField("superField");
+		var field1 = ClassWithAnnotatedFields.class.getDeclaredField("field1");
+		var field3 = ClassWithAnnotatedFields.class.getDeclaredField("field3");
+		var superField = SuperclassWithAnnotatedField.class.getDeclaredField("superField");
 
-		List<Field> fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation1.class, isStringField,
-			BOTTOM_UP);
+		var fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation1.class, isStringField, BOTTOM_UP);
 
 		assertEquals(3, fields.size());
 		assertThat(fields.subList(0, 2)).containsOnly(field1, field3);
@@ -398,12 +439,11 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotatedFieldsForAnnotationUsedInClassAndSuperclassHierarchyDown() throws Exception {
-		Field field1 = ClassWithAnnotatedFields.class.getDeclaredField("field1");
-		Field field3 = ClassWithAnnotatedFields.class.getDeclaredField("field3");
-		Field superField = SuperclassWithAnnotatedField.class.getDeclaredField("superField");
+		var field1 = ClassWithAnnotatedFields.class.getDeclaredField("field1");
+		var field3 = ClassWithAnnotatedFields.class.getDeclaredField("field3");
+		var superField = SuperclassWithAnnotatedField.class.getDeclaredField("superField");
 
-		List<Field> fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation1.class, isStringField,
-			TOP_DOWN);
+		var fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation1.class, isStringField, TOP_DOWN);
 
 		assertEquals(3, fields.size());
 		assertEquals(superField, fields.get(0));
@@ -412,10 +452,9 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findAnnotatedFieldsForAnnotationUsedInInterface() throws Exception {
-		Field interfaceField = InterfaceWithAnnotatedField.class.getDeclaredField("interfaceField");
+		var interfaceField = InterfaceWithAnnotatedField.class.getDeclaredField("interfaceField");
 
-		List<Field> fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation3.class, isStringField,
-			BOTTOM_UP);
+		var fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation3.class, isStringField, BOTTOM_UP);
 
 		assertThat(fields).containsExactly(interfaceField);
 	}
@@ -423,11 +462,11 @@ class AnnotationUtilsTests {
 	@Test
 	void findAnnotatedFieldsForShadowedFields() throws Exception {
 		Class<?> clazz = ClassWithShadowedAnnotatedFields.class;
-		Field interfaceField = clazz.getDeclaredField("interfaceField");
-		Field superField = clazz.getDeclaredField("superField");
-		Field field1 = clazz.getDeclaredField("field1");
-		Field field2 = clazz.getDeclaredField("field2");
-		Field field3 = clazz.getDeclaredField("field3");
+		var interfaceField = clazz.getDeclaredField("interfaceField");
+		var superField = clazz.getDeclaredField("superField");
+		var field1 = clazz.getDeclaredField("field1");
+		var field2 = clazz.getDeclaredField("field2");
+		var field3 = clazz.getDeclaredField("field3");
 
 		assertThat(findShadowingAnnotatedFields(Annotation1.class)).containsExactly(superField, field1, field3);
 		assertThat(findShadowingAnnotatedFields(Annotation2.class)).containsExactly(field2, field3);
@@ -460,44 +499,43 @@ class AnnotationUtilsTests {
 
 	@Test
 	void findPublicAnnotatedFieldsForPrivateField() {
-		List<Field> fields = findPublicAnnotatedFields(getClass(), Boolean.class, Annotation1.class);
+		var fields = findPublicAnnotatedFields(getClass(), Boolean.class, Annotation1.class);
 		assertNotNull(fields);
 		assertEquals(0, fields.size());
 	}
 
 	@Test
 	void findPublicAnnotatedFieldsForDirectlyAnnotatedFieldOfWrongFieldType() {
-		List<Field> fields = findPublicAnnotatedFields(getClass(), BigDecimal.class, Annotation1.class);
+		var fields = findPublicAnnotatedFields(getClass(), BigDecimal.class, Annotation1.class);
 		assertNotNull(fields);
 		assertEquals(0, fields.size());
 	}
 
 	@Test
 	void findPublicAnnotatedFieldsForDirectlyAnnotatedField() {
-		List<Field> fields = findPublicAnnotatedFields(getClass(), String.class, Annotation1.class);
+		var fields = findPublicAnnotatedFields(getClass(), String.class, Annotation1.class);
 		assertNotNull(fields);
-		assertIterableEquals(asList("directlyAnnotatedField"), asNames(fields));
+		assertIterableEquals(List.of("directlyAnnotatedField"), asNames(fields));
 	}
 
 	@Test
 	void findPublicAnnotatedFieldsForMetaAnnotatedField() {
-		List<Field> fields = findPublicAnnotatedFields(getClass(), Number.class, Annotation1.class);
+		var fields = findPublicAnnotatedFields(getClass(), Number.class, Annotation1.class);
 		assertNotNull(fields);
 		assertEquals(1, fields.size());
-		assertIterableEquals(asList("metaAnnotatedField"), asNames(fields));
+		assertIterableEquals(List.of("metaAnnotatedField"), asNames(fields));
 	}
 
 	@Test
 	void findPublicAnnotatedFieldsForDirectlyAnnotatedFieldInInterface() {
-		List<Field> fields = findPublicAnnotatedFields(InterfaceWithAnnotatedFields.class, String.class,
-			Annotation1.class);
+		var fields = findPublicAnnotatedFields(InterfaceWithAnnotatedFields.class, String.class, Annotation1.class);
 		assertNotNull(fields);
-		assertIterableEquals(asList("foo"), asNames(fields));
+		assertIterableEquals(List.of("foo"), asNames(fields));
 	}
 
 	@Test
 	void findPublicAnnotatedFieldsForDirectlyAnnotatedFieldsInClassAndInterface() {
-		List<Field> fields = findPublicAnnotatedFields(ClassWithPublicAnnotatedFieldsFromInterface.class, String.class,
+		var fields = findPublicAnnotatedFields(ClassWithPublicAnnotatedFieldsFromInterface.class, String.class,
 			Annotation1.class);
 		assertNotNull(fields);
 		assertThat(asNames(fields)).containsExactlyInAnyOrder("foo", "bar");
@@ -611,14 +649,25 @@ class AnnotationUtilsTests {
 	@Retention(RetentionPolicy.RUNTIME)
 	@Inherited
 	@ExtendWith("foo")
+	@Repeatable(FooExtensions.class)
 	@interface ExtendWithFoo {
+
+		String info() default "";
+	}
+
+	@Target({ ElementType.TYPE, ElementType.METHOD })
+	@Retention(RetentionPolicy.RUNTIME)
+	@Inherited
+	@interface FooExtensions {
+
+		ExtendWithFoo[] value();
 	}
 
 	@Target({ ElementType.TYPE, ElementType.METHOD })
 	@Retention(RetentionPolicy.RUNTIME)
 	// Intentionally NOT @Inherited in order to ensure that the algorithm for
 	// findRepeatableAnnotations() in fact lives up to the claims in the
-	// JavaDoc regarding searching for repeatable annotations with implicit
+	// Javadoc regarding searching for repeatable annotations with implicit
 	// "inheritance" if the repeatable annotation is @Inherited but the
 	// custom composed annotation is not @Inherited.
 	// @Inherited
@@ -632,6 +681,12 @@ class AnnotationUtilsTests {
 
 	@Annotation1
 	static class Annotation1Class {
+		class InnerClass {
+			class InnerInnerClass {
+			}
+		}
+		static class NestedClass {
+		}
 	}
 
 	@Annotation2
@@ -655,6 +710,11 @@ class AnnotationUtilsTests {
 		@ComposedAnnotation
 		void method() {
 		}
+
+		class InnerClass {
+			class InnerInnerClass {
+			}
+		}
 	}
 
 	@InheritedComposedAnnotation
@@ -767,6 +827,15 @@ class AnnotationUtilsTests {
 	static class SingleComposedExtensionClass {
 	}
 
+	@ExtendWithFoo(info = "A")
+	@ExtendWithFoo(info = "B")
+	static class MultipleFoos1 {
+	}
+
+	@FooExtensions({ @ExtendWithFoo(info = "A"), @ExtendWithFoo(info = "B") })
+	static class MultipleFoos2 {
+	}
+
 	static class SubSingleComposedExtensionClass extends SingleComposedExtensionClass {
 	}
 
